<?php
/*
# Google Map API for PmWiki 
# =========================
# 
# Copyright Statement 
# -------------------
# 
# Copyright (c) 2006, Benjamin C. Wilson. All Rights Reserved. 
# 
# Read the documentation.txt accompanying this software package for licensing
# and installation and usage instructions.
#
*/
define('GMAPATH', dirname(__FILE__) . '/');
include_once(GMAPATH.'library/php-local-browscap.php');

# SDV()
SDV($GmaDebug, 0);
SDV($GmaDefaults, array(
    'ctrl' => 'large,overview,type',
    'view' => 'normal',
    'height' => '300px',
    'width' => '500px',
    'float' => '',
    'color' => '#808',
    'thickness' => 2,
    'opacity' => 0.7,
    'zoom' => 0
    )
);
SDV($GmaKey, '');
# Internals.
$GmaDebugMsg = array();
$GmaHTML = '';
$GmaIEFix = '';
$GmaLines = array();
$GmaPoints = array();
$GmaPointPopulation = 0;
$GmaVersion = '2.0.0';
# Markup
# * (:gma-map [options]:)
# * (:gma-point lat lon [options]:)
# * (:gma-line lat lon [options]:)
Markup('gma', '_begin', '/\(:gma-(\S+)\s?(.*?):\)/ie',"gmaMarkup('$1','$2');");
class GmaLine
{
  function GmaLine($id) {
    global $GmaLines, $GmaDefaults;
    $this->id = $id;
    $this->color = $GmaDefaults['color'];
    $this->opacity = $GmaDefaults['opacity'];
    $this->points = array();
    $this->thickness = $GmaDefaults['thickness'];
  }
  function addPoint($lat, $lon) {
    if ($lat && $lon) array_push($this->points, "new GLatLng($lat, $lon)");
  }
  function line($map_id='map') {
    $id = $this->id;
    $pad = str_pad('', 26 + strlen($id), ' ', STR_PAD_LEFT);
    $points = '['.join(",\n$pad", $this->points).']';
    $opacity = ($this->opacity > 1) 
        ? sprintf("%0.2f", $this->opacity / 100)
        : $this->opacity;
    $opacity = ($opacity > 1) ? 1.0 : $opacity;
    $options = join(', ', array(
                    $points,
                    $this->color(),
                    $this->thickness,
                    $opacity
                ));
    $this->overlay = "    $map_id.addOverlay($id);\n";
    return "    var $id = new GPolyline($options);\n";
  }
  function color() {
    // This method returns only web-safe colors.
    $color = preg_replace("/\\\'/",'',$this->color);
    if (0 === strpos($color, '#')) $color = substr($color, 1);
    // break into hex 3-tuple
    $cutpoint = ceil(strlen($color) / 2) - 1;
    $rgb = explode(':', wordwrap($color, $cutpoint, ':', $cutpoint), 3);

    $out = '';
    foreach($rgb as $r) {
        if (strlen($r) == 1) $r .= $r; # Expand single value colors.
        $r = (isset($r)) ? hexdec($r) : 0; # Hex to Dec.
        $r = (round($r/51) * 51); # Make web-safe
        $out .= str_pad(dechex($r), 2, '0', STR_PAD_LEFT); # Code color.
    }
    return "'#$out'";
  }
}

class GmaMap
{
    var $map_views = array(
      'hyb' => 'G_HYBRID_MAP',
      'nor' => 'G_NORMAL_MAP',
      'sat' => 'G_SATELLITE_MAP',
    );
    var $controls_js = array(
      'large' => 'new GLargeMapControl()',
      'maptype' => 'new GMapTypeControl()', # allow toggle b/w map types.
      'overview' => 'new GOverviewMapControl()', # allow map controls.
      'small' => 'new GSmallMapControl()',
      'type' => 'new GMapTypeControl()',
    );
    var $controls = array();
    function GmaMap($id='map') {
      global $GmaDefaults;
      $this->id = $id;
      $this->height = $GmaDefaults['height'];
      $this->view = $GmaDefaults['view'];
      $this->width = $GmaDefaults['width'];
      $this->zoom = $GmaDefaults['zoom'];
      $this->setControl((array) explode(',', $GmaDefaults['ctrl']));
      $this->lat = "null";
      $this->lon = "null";
    }
    function setControl($controls) {
      foreach ((array) $controls as $c) {
        if ($c[0] == '-' && array_key_exists(substr($c,1), $this->controls)) {
          unset($this->controls[substr($c,1)]);
        }
        else {
          $this->controls[$c] = 1;
        }
      }
    }
    function getControls() {
        $ret = '';
        $id = $this->id;
        foreach (array_keys($this->controls) as $c) {
          if ($c == 'small' && array_key_exists('large', $this->controls)){
            unset($this->controls[$c]);
          }
          elseif ($c == 'large' && array_key_exists('small', $this->controls)){
            unset($this->controls[$c]);
          }
          elseif ($c[0] != '-') {
            $c = $this->controls_js[$c];
            $ret .= "    $id.addControl($c);\n";
          }
        }
        return $ret;
    }
    function view() {
        return $this->map_views[strtolower(substr($this->view,0,3))];
    }
}
class GmaPoint
{
  var $directions = '';
  var $icon = '';
  var $lat = 0.0;
  var $lon = 0.0;
  var $message = '';
  var $title = '';
  function GmaPoint($lat='', $lon='') {
    global $GmaPointPopulation;
    $this->id = $GmaPointPopulation++;
    if ($lat && $lon) { $this->lat = $lat; $this->lon = $lon; }
  }
  function icon() { return $this->_quote($this->icon);}
  function point($map_id='map') {
    $icon = $this->icon();
    $lat = $this->lat;
    $lon = $this->lon;
    $text = $this->message();
    $title = $this->_quote($this->title);
    return "    addGmaPoint($map_id,$lat,$lon,$title,$text,$icon);\n";
  }
  function _quote($i) { return ($i) ? $this->_jsstrip("'$i'") : 'null'; }
  function _jsstrip($i) { return $i;} # TODO: Strip Javascript.
  function message() {
    if ($this->message) {
      $m = $this->message;
      $m = preg_replace('/&lt;/','<', $m);
      $m = preg_replace('/&gt;/','>', $m);
    }
    $m = $this->_quote($m);
    return $m;
  }
  function anchor($anchor,$mapid='map') {
    if (!$anchor) { return ''; }
    $anchor = $this->_jsstrip($anchor);
    $id = $this->id;
    $link =  "<a name='{$anchor}'></a>"
            ."<a href='javascript:makeGmalink({$mapid},{$id});'>{$anchor}</a>";
    $this->message .= "<a href=\'#$anchor\'>$anchor</a>";
    return "$link";
  }
} // End Class GmaPoint
function gmaBrowserFix() {
}
function gmaCleanup() {
  global $HTMLHeaderFmt, $HTMLStylesFmt, $HTMLFooterFmt;
  global $GmaEnable, $GmaVersion, $GmaScript;
  global $GmaMap, $GmaPoints, $GmaLines;
  global $GmaKey, $GmaDebugMsg, $GmaDefaults;

  GmaDoIEFix();
  #--------------------------------
  # Set the HTML
  $height = $GmaMap->height;
  $width = $GmaMap->width;
  $clat = $GmaMap->lat;
  $clon = $GmaMap->lon;
  $float = ($GmaMap->float) ? "float: {$GmaMap->float};" : '';
  $map_id = $GmaMap->id;

  $HTMLStylesFmt['gmap_api'] 
     = "div#$map_id{ height: $height; width:$width;$float}";
  $HTMLHeaderFmt[] 
     =  '<style type=\'text/css\'>v:* { behavior:url(#default#VML); }</style>'
       ."\n<script src='http://maps.google.com/maps?file=api&v=2&key="
       .$GmaKey."' type='text/javascript'></script>";
  $HTMLHeaderFmt[] 
     =  "\n<script language='javascript' src='\$FarmPubDirUrl/scripts/gmaJs.js'>"
       ."</script>\n"; 

  GmaDebug(print_r($GmaMap, 1));
  $controls = $GmaMap->getControls();
  $view = $GmaMap->view();
  $zoom = $GmaMap->zoom or 'null';
  $overlay = '';
  $points = '';
  $lines = '';
  foreach ($GmaPoints as $p) { $points .= $p->point($map_id); }
  foreach ($GmaLines as $l) { 
      $lines .= $l->line($map_id); 
      $overlay .= $l->overlay;
  }
  $debug = ($GmaDebug) ? implode("\n", (array) $GmaDebugMsg) : '';
  $defaults = $GmaDefaults; 
  $defaults = preg_replace('/Array|\(\\n|\)/', '', print_r($defaults, 1));

  $HTMLFooterFmt[] =<<<GMASCRIPT
  $debug
  <!-- GMA Site Default Controls:
  $defaults -->
  <script type="text/javascript">
  //<![CDATA[
  // Copyright (c) 2006, Benjamin C. Wilson. All Rights Reserved.
  // Google Map API for PmWiki, $GmaVersion.
  // This copyright statement must accompany this script.
  if (GBrowserIsCompatible()) {
    var from_htmls = [];
    var htmls = [];
    var markers = [];
    var points = [];
    var to_htmls = [];
    var i = 0;
    var $map_id = new GMap2(document.getElementById('$map_id'));
    var bounds = new GLatLngBounds;
$points
$lines
    setGmaMapCenter($map_id, $view, $zoom, $clat, $clon);
    $map_id.addControl(new GScaleControl());
$controls
$overlay
    doGmaOverlay($map_id);
  }
</script>
GMASCRIPT;
}
function gmaDebug($m) {
  // (null) gmaDebug(string);
  //
  // Packs end-of-cycle debugging information.
  global $GmaDebug, $GmaDebugMsg;
  array_push($GmaDebugMsg, "<pre>$m</pre>");
}
function gmaDefaults($opt, $def, $orig='') {
  // (mixed) gmaDefaults(option, site-default);
  //
  // This function overrides site default variable.
  if (!($opt || $def || $org)) return '';
  $val = ($opt && !$orig) ? $opt : $def;
  $val = ($orig) ? $orig : ($opt) ? $opt : $def;
  return $val;
}
function gmaLLZError($type, $lat, $lon) {
  // (bool) gmaLLZError(type, lat, lon);
  //
  // function flags error when the type requires a lattitude or longitude.
  if ($type == 'map') return 0;
  if (!$lat && !$lon) return 1;
}
function gmaMarkup($type, $args) {
  // (string) gmaMarkup(type, args);
  //
  // This function converts all PmWiki markup into GMA objects and attributes
  // and reconciles changes to the site-default.

  global $GmaDefaults, $GmaDebug;
  gmaSpoofLocale();
  $ret = '';

  $opts = parseArgs($args);
  GmaDebug("OPTIONS:".print_r($opts,1));
  GmaDebug("CONTROLS:".print_r($controls,1));

  // GMA accepts either ZIP code or lat/lon.
  if ($opts['zip'])
    list($opts['lat'], $opts['lon']) = gmaZIP2LatLon($opts['zip']);
  if (gmaLLZError($type, $opts['lat'], $opts['lon']))
    return 'GMA Error: No Lat/Lon or ZIP given (or incomplete).';

  // This switch handles the behavior of the various GMA Types.
  switch($type) {
    // Gma Type: Line "(:gma-line lat lon [options]:)"
    case ('line'):
        global $GmaLines;
        $lid = GmaDefaults('line'.$opts['id'], 'line0');
        if (!array_key_exists($lid, $GmaLines)) {
          $GmaLines[$lid] = new GmaLine($lid);
        }
        $line = &$GmaLines[$lid];
        if ($opts['color']) $line->color = $opts['color'];
        if ($opts['thickness']) $line->color = $opts['thickness'];
        if ($opts['opacity']) $line->opacity = $opts['opacity'];

        $line->addPoint($opts['lat'], $opts['lon']);
        $ret .= ($GmaDebug) ?  "<pre>".Keep(print_r($line,1)) : '';
        break;

    // Gma Type: Map "(:gma-map [options]:)"
    case ('map'):
        global $GmaMap;
        global $MarkupFrame;
        // Create The Map Object and set attributes.
        $GmaMap = ($opts['id']) ? new GmaMap($opts['id']) : new GmaMap();
        if ($opts['ctrl']) $controls = (array) explode(',', $opts['ctrl']);
        $GmaMap->setControl($controls);
        if ($opts['zoom']) $GmaMap->zoom = $opts['zoom'];
        if ($opts['view']) $GmaMap->view = $opts['view'];
        if ($opts['height']) $GmaMap->height = $opts['height'];
        if ($opts['width']) $GmaMap->width = $opts['width'];
        if ($opts['lat'] && $opts['lon']) {
            $GmaMap->lat = $opts['lat'];
            $GmaMap->lon = $opts['lon'];
        }

        // Trigger the end-of-markup addition of the Javascript,
        // and give the map's target.
        if (!$MarkupFrame[0]['posteval']['mymarkup'])
          $MarkupFrame[0]['posteval']['mymarkup'] = 'gmaCleanup();';
        $ret = Keep("<div id='{$GmaMap->id}'></div>");
        $ret .= ($GmaDebug) ?  "<pre>".Keep(print_r($GmaMap,1)) : '';
        break;

    // Gma Type: Point "(:gma-point lat lon [options]:)"
    case ('point'):
        global $GmaPoints;
        // Build the Point object.
        $point = new GmaPoint($opts['lat'], $opts['lon']);
        $point->directions = gmaDefaults($opts['daddr'], $GmapDaddr);
        $point->icon = gmaDefaults($opts['icon'], '');
        $point->message .= gmaDefaults($opts['text'], '');
        if ($opts['link']) $ret .= $point->anchor($opts['link']);
        // Add the point to the Collection.
        array_push($GmaPoints, $point);
        $ret .= ($GmaDebug) ?  "<pre>".Keep(print_r($point,1)) : '';
        break;
    // Gma Type: (Invalid)
    default:
        $ret = "GMA Error: Type Unknown ($type)";
  }
  gmaSpoofLocale();
  return $ret;
}
function gmaSpoofLocale() {
    // (null) gmaSpoofLocale(null);
    //
    // Provided by HelgeLarsen. This allows for non-US locales to behave as
    // normal. I moved it into a function for smoother toggle, but this may be
    // OBE.
    global $GmaHoldLocale, $GmaLocaleToggle;
    if ($GmaLocaleToggle) {
      setlocale(LC_NUMERIC,$GmapHoldLocale);
    }
    else {
      $GmapHoldLocale = setlocale(LC_NUMERIC,'0');
      setlocale(LC_NUMERIC,'en_US');
    }
    $GmaLocaleToggle = ($GmaLocalToggle) ? 0 : 1;
}
function gmaZIP2LatLon($z) {
  // (array) gmaZIP2LatLon(int);
  //
  // This function allows a site to use ZIP codes for lat-lon, when configured.
  // Specifically, the function checks for the ZIP file and then greps the
  // answer.
  $zip_file = GMAPATH.'/library/gma_zip.csv';
  if (file_exists($zip_file) && $zip_file != '') {
    exec("grep '^$z' $zip_file", $r);
    $x = split(',', $r[0]);
    list($lat, $lon) = array($x[1], $x[2]);
    $lon = preg_replace('/^(.)0/', "$1", $lon);
    return array($lat, $lon);
  }
  return array(null,null); # No zip file.
}
function GmaDoIEFix() {
  global $GmapIEFix;
  if (get_cfg_var('browscap')) {
   $browser=get_browser(); //If available, use PHP native function
  }
  else {
   require_once(GMAPATH."library/php-local-browscap.php");
   $browser=get_browser_local(null, false, GMAPATH."/library/browscap.ini");
  }
  $GmapIEFix = (preg_match('/IE/', $browser->browser))
    ? " xmlns='http://www.w3.org/1999/xhtml' xmlns:v='urn:schemas-microsoft-com:vml'"
    : '';
}

